@@ -1,5 +1,4 @@
 import contextlib
-import json
 import logging
 import subprocess
 import tempfile
@@ -7,23 +6,28 @@ import time
 from enum import Enum
 from itertools import cycle
 from pathlib import Path
-from typing import Any, Dict, Generator, List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import fastar
 import rignore
 import typer
 from httpx import Client
-from pydantic import BaseModel, EmailStr, TypeAdapter, ValidationError
+from pydantic import BaseModel, EmailStr, ValidationError
 from rich.text import Text
 from rich_toolkit import RichToolkit
 from rich_toolkit.menu import Option
 from typing_extensions import Annotated
 
 from fastapi_cloud_cli.commands.login import login
-from fastapi_cloud_cli.utils.api import APIClient
+from fastapi_cloud_cli.utils.api import APIClient, BuildLogError, TooManyRetriesError
 from fastapi_cloud_cli.utils.apps import AppConfig, get_app_config, write_app_config
 from fastapi_cloud_cli.utils.auth import is_logged_in
 from fastapi_cloud_cli.utils.cli import get_rich_toolkit, handle_http_errors
+from fastapi_cloud_cli.utils.pydantic_compat import (
+    TypeAdapter,
+    model_dump,
+    model_validate,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -91,7 +95,7 @@ def _get_teams() -> List[Team]:
 
         data = response.json()["data"]
 
-    return [Team.model_validate(team) for team in data]
+    return [model_validate(Team, team) for team in data]
 
 
 class AppResponse(BaseModel):
@@ -108,7 +112,7 @@ def _create_app(team_id: str, app_name: str) -> AppResponse:
 
         response.raise_for_status()
 
-        return AppResponse.model_validate(response.json())
+        return model_validate(AppResponse, response.json())
 
 
 class DeploymentStatus(str, Enum):
@@ -161,7 +165,7 @@ def _create_deployment(app_id: str) -> CreateDeploymentResponse:
         response = client.post(f"/apps/{app_id}/deployments/")
         response.raise_for_status()
 
-        return CreateDeploymentResponse.model_validate(response.json())
+        return model_validate(CreateDeploymentResponse, response.json())
 
 
 class RequestUploadResponse(BaseModel):
@@ -186,7 +190,7 @@ def _upload_deployment(deployment_id: str, archive_path: Path) -> None:
         response = fastapi_client.post(f"/deployments/{deployment_id}/upload")
         response.raise_for_status()
 
-        upload_data = RequestUploadResponse.model_validate(response.json())
+        upload_data = model_validate(RequestUploadResponse, response.json())
         logger.debug("Received upload URL: %s", upload_data.url)
 
         # Upload the archive
@@ -221,7 +225,7 @@ def _get_app(app_slug: str) -> Optional[AppResponse]:
 
         data = response.json()
 
-    return AppResponse.model_validate(data)
+    return model_validate(AppResponse, data)
 
 
 def _get_apps(team_id: str) -> List[AppResponse]:
@@ -231,24 +235,14 @@ def _get_apps(team_id: str) -> List[AppResponse]:
 
         data = response.json()["data"]
 
-    return [AppResponse.model_validate(app) for app in data]
-
-
-def _stream_build_logs(deployment_id: str) -> Generator[str, None, None]:
-    with APIClient() as client:
-        with client.stream(
-            "GET", f"/deployments/{deployment_id}/build-logs", timeout=60
-        ) as response:
-            response.raise_for_status()
-
-            yield from response.iter_lines()
+    return [model_validate(AppResponse, app) for app in data]
 
 
 WAITING_MESSAGES = [
     "🚀 Preparing for liftoff! Almost there...",
     "👹 Sneaking past the dependency gremlins... Don't wake them up!",
     "🤏 Squishing code into a tiny digital sandwich. Nom nom nom.",
-    "📉 Server space running low. Time to delete those cat videos?",
+    "🐱 Removing cat videos from our servers to free up space.",
     "🐢 Uploading at blazing speeds of 1 byte per hour. Patience, young padawan.",
     "🔌 Connecting to server... Please stand by while we argue with the firewall.",
     "💥 Oops! We've angered the Python God. Sacrificing a rubber duck to appease it.",
@@ -358,17 +352,15 @@ def _wait_for_deployment(
 
     with toolkit.progress(
         next(messages), inline_logs=True, lines_to_show=20
-    ) as progress:
-        with handle_http_errors(progress=progress):
-            for line in _stream_build_logs(deployment.id):
+    ) as progress, APIClient() as client:
+        try:
+            for log in client.stream_build_logs(deployment.id):
                 time_elapsed = time.monotonic() - started_at
 
-                data = json.loads(line)
+                if log.type == "message":
+                    progress.log(Text.from_ansi(log.message.rstrip()))
 
-                if "message" in data:
-                    progress.log(Text.from_ansi(data["message"].rstrip()))
-
-                if data.get("type") == "complete":
+                if log.type == "complete":
                     progress.log("")
                     progress.log(
                         f"🐔 Ready the chicken! Your app is ready at [link={deployment.url}]{deployment.url}[/link]"
@@ -382,7 +374,7 @@ def _wait_for_deployment(
 
                     break
 
-                if data.get("type") == "failed":
+                if log.type == "failed":
                     progress.log("")
                     progress.log(
                         f"😔 Oh no! Something went wrong. Check out the logs at [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]"
@@ -390,12 +382,20 @@ def _wait_for_deployment(
                     raise typer.Exit(1)
 
                 if time_elapsed > 30:
-                    messages = cycle(LONG_WAIT_MESSAGES)  # pragma: no cover
+                    messages = cycle(LONG_WAIT_MESSAGES)
 
                 if (time.monotonic() - last_message_changed_at) > 2:
-                    progress.title = next(messages)  # pragma: no cover
+                    progress.title = next(messages)
 
-                    last_message_changed_at = time.monotonic()  # pragma: no cover
+                    last_message_changed_at = time.monotonic()
+
+        except (BuildLogError, TooManyRetriesError) as e:
+            logger.error("Build log streaming failed: %s", e)
+            toolkit.print_line()
+            toolkit.print(
+                f"⚠️  Unable to stream build logs. Check the dashboard for status: [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]"
+            )
+            raise typer.Exit(1) from e
 
 
 class SignupToWaitingList(BaseModel):
@@ -416,9 +416,7 @@ def _send_waitlist_form(
     with toolkit.progress("Sending your request...") as progress:
         with APIClient() as client:
             with handle_http_errors(progress):
-                response = client.post(
-                    "/users/waiting-list", json=result.model_dump(mode="json")
-                )
+                response = client.post("/users/waiting-list", json=model_dump(result))
 
                 response.raise_for_status()
 
@@ -443,7 +441,7 @@ def _waitlist_form(toolkit: RichToolkit) -> None:
 
     toolkit.print_line()
 
-    result = SignupToWaitingList(email=email)
+    result = model_validate(SignupToWaitingList, {"email": email})
 
     if toolkit.confirm(
         "Do you want to get access faster by giving us more information?",
@@ -467,11 +465,12 @@ def _waitlist_form(toolkit: RichToolkit) -> None:
         result = form.run()  # type: ignore
 
         try:
-            result = SignupToWaitingList.model_validate(
+            result = model_validate(
+                SignupToWaitingList,
                 {
                     "email": email,
                     **result,  # type: ignore
-                }
+                },
             )
         except ValidationError:
             toolkit.print(
