@@ -11,6 +11,7 @@ from fastapi_cloud_cli.utils.apps import get_app_config
 from fastapi_cloud_cli.utils.auth import is_logged_in
 from fastapi_cloud_cli.utils.cli import get_rich_toolkit, handle_http_errors
 from fastapi_cloud_cli.utils.env import validate_environment_variable_name
+from fastapi_cloud_cli.utils.pydantic_compat import model_validate
 
 logger = logging.getLogger(__name__)
 
@@ -29,7 +30,7 @@ def _get_environment_variables(app_id: str) -> EnvironmentVariableResponse:
         response = client.get(f"/apps/{app_id}/environment-variables/")
         response.raise_for_status()
 
-        return EnvironmentVariableResponse.model_validate(response.json())
+        return model_validate(EnvironmentVariableResponse, response.json())
 
 
 def _delete_environment_variable(app_id: str, name: str) -> bool:
